import os
import logging

from opentelemetry._logs import set_logger_provider
from opentelemetry.exporter.otlp.proto.grpc._log_exporter import (
    OTLPLogExporter,
)
from opentelemetry.sdk._logs import LoggerProvider, LoggingHandler
from opentelemetry.sdk._logs.export import BatchLogRecordProcessor
from opentelemetry.sdk.resources import Resource

mw_agent_target = os.environ.get('MW_AGENT_SERVICE', '127.0.0.1')


def log_handler(project_name, service_name):
    resource_json = {
        "service.name": service_name,
    }

    if project_name is not None:
        resource_json["project.name"] = project_name

    logger_provider = LoggerProvider(
        resource=Resource.create(resource_json),
    )
    set_logger_provider(logger_provider)
    logger_provider.add_log_record_processor(BatchLogRecordProcessor(
        OTLPLogExporter(insecure=True, endpoint=mw_agent_target+":9319")
    ))
    return LoggingHandler(level=logging.NOTSET, logger_provider=logger_provider)



import os
import psutil
import logging
from middleware.config import config

mw_agent_target = os.environ.get('MW_AGENT_SERVICE', '127.0.0.1')


class MwTracker:
    def __init__(self):

        # resource_attributes = os.environ.get("OTEL_RESOURCE_ATTRIBUTES")
        # _service_name = os.environ.get("OTEL_SERVICE_NAME")
        # _project_name = self._get_project_name(resource_attributes)

        self.project_name = config.project_name
        self.service_name = config.service_name
        self.access_token = config.access_token or ""

        if config.collect_metrics: self.collect_metrics()
        if config.collect_traces: self.collect_traces()
        if config.collect_logs: self.collect_logs()
        if config.collect_profiling and not psutil.WINDOWS: self.collect_profiling()

    def collect_metrics(self):
        from ._meter import collect_metrics
        collect_metrics()

    def collect_traces(self):
        os.environ['OTEL_PYTHON_TRACER_PROVIDER'] = 'sdk_tracer_provider'
        from ._tracer import collect_traces
        collect_traces()

    def collect_profiling(self):
        from ._profiler import collect_profiling
        collect_profiling(self.service_name, self.access_token)

    def collect_logs(self):
        from ._logger import log_handler
        handler = log_handler(self.project_name, self.service_name)
        logging.getLogger().addHandler(handler)
        logging.setLogRecordFactory(self._set_custom_log_attr)

    def record_error(self, error):
        from ._tracer import record_error
        record_error(error)

    def django_instrument(self):
        if config.collect_traces:
            from opentelemetry.instrumentation.django import DjangoInstrumentor
            DjangoInstrumentor().instrument()

    def _set_custom_log_attr(self, *args, **kwargs):
        record = logging.LogRecord(*args, **kwargs)
        update_json = {"service.name": self.service_name, "mw.app.lang": "python"}
        if self.project_name: update_json["project.name"] = self.project_name
        record.__dict__.update(update_json)
        return record

    def _get_project_name(self, resource_attributes) -> str:
        project_name = None
        if resource_attributes is not None:
            attributes_list = resource_attributes.split(',')
            for attribute in attributes_list:
                key, value = attribute.split('=')
                if key == 'project.name':
                    project_name = value

        return project_name



import subprocess
from middleware.config import config
from middleware.installer import command
from middleware.installer._bootstrap import _bootstrap


@command('run', '...',
         """Executes the command line but forces the initialization of the agent
         automatically at startup.

         If using an agent configuration file, the path to the file should be
         supplied by the environment variable MIDDLEWARE_CONFIG_FILE.""")
def run(args):
    if len(args) == 0:
        print("Expected args are missing.")
        return

    exporters = [
        "--traces_exporter otlp" if config.collect_traces else "--traces_exporter none",
        "--metrics_exporter otlp" if config.collect_metrics else "--metrics_exporter none",
        "--logs_exporter otlp" if config.collect_logs else "--logs_exporter none"
    ]

    # cmd1 = "middleware-bootstrap -a install"
    cmd2 = f"middleware-instrument {' '.join(exporters)} \
    --tracer_provider sdk_tracer_provider \
    --exporter_otlp_endpoint {config.exporter_otlp_endpoint} \
    --resource_attributes={config.resource_attributes} \
    --service_name {config.service_name} \
    --propagators {config.otel_propagators} {' '.join(args)}"

    # try:
    #     subprocess.run(cmd1, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=True, check=True)
    # except subprocess.CalledProcessError as e:
    #     print(f"Failed bootstrapping with error: {e}")

    _bootstrap()

    try:
        subprocess.run(cmd2, shell=True, check=True)
    except subprocess.CalledProcessError as e:
        print(f"Failed instrumenting with error: {e}")



import os
import sys
import configparser


class Config:
    def __init__(self):
        if len(sys.argv) > 1 and sys.argv[1] == "help":
            return

        config_file = os.environ.get("MIDDLEWARE_CONFIG_FILE", os.path.join(os.getcwd(), 'middleware.ini'))
        if not os.path.exists(config_file):
            exit_with_error(f"File not found: {config_file}")

        self.config = configparser.ConfigParser()
        try:
            self.config.read(config_file)
        except configparser.Error as e:
            exit_with_error(f"Error reading configuration file: {e}")

        pid = os.getpid()
        self.project_name = self.get_config("middleware.common", "project_name", None)
        self.service_name = self.get_config("middleware.common", "service_name", f"Service-{pid}")
        self.access_token = self.get_config("middleware.common", "access_token", "")
        self.collect_traces = self.get_config_boolean("middleware.common", "collect_traces", True)
        self.collect_metrics = self.get_config_boolean("middleware.common", "collect_metrics", False)
        self.collect_logs = self.get_config_boolean("middleware.common", "collect_logs", False)
        self.collect_profiling = self.get_config_boolean("middleware.common", "collect_profiling", False)
        self.otel_propagators = self.get_config("middleware.common", "otel_propagators", "b3")

        project_name_attr = f"project.name={self.project_name}," if self.project_name else ""
        source_service_url = self.get_config("middleware.common", "mw_agent_service", "localhost")

        mw_agent_service = os.environ.get("MW_AGENT_SERVICE", None)
        if mw_agent_service is not None and mw_agent_service != "":
            source_service_url = mw_agent_service

        self.exporter_otlp_endpoint = f"http://{source_service_url}:9319"
        self.resource_attributes = f"{project_name_attr}mw.app.lang=python,runtime.metrics.python=true"

    def get_config(self, section, key, default):
        return self.config.get(section, key, fallback=default)

    def get_config_boolean(self, section, key, default):
        return self.config.getboolean(section, key, fallback=default)


def exit_with_error(message):
    print(message)
    sys.exit(1)


config = Config()



import subprocess
import pkg_resources
import sys

# This command scans the list of packages currently present in your active 'site-packages' directory
# and automatically deploys the relevant instrumentation libraries associated with these packages, if applicable.
#
# For instance, if you have previously installed the 'flask' package,
# this command will install 'opentelemetry-instrumentation-flask' on your behalf.
def _pip_install(package):
    # explicit upgrade strategy to override potential pip config
    subprocess.check_call(
        [
            sys.executable,
            "-m",
            "pip",
            "install",
            "-U",
            "--upgrade-strategy",
            "only-if-needed",
            package,
        ]
    )

def _bootstrap():

    cmd = "middleware-bootstrap"

    try:
        result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True, text=True)
        output = result.stdout
    except subprocess.CalledProcessError as e:
        print(f"Error running middleware-bootstrap: {e}")
        output = ""

    libraries = output.splitlines()

    for library in libraries:
        library_name, library_version = library.split("==", 1)

        try:
            installed_version = pkg_resources.get_distribution(library_name).version
            if installed_version != library_version:
                # The library is not updated, so update it
                try:
                    _pip_install(library)
                    # subprocess.run(["pip", "uninstall", "-y", library_name], check=True)
                    # subprocess.run(["pip", "install", library], check=True)
                    print(f"{library_name} has been updated to version {library_version}.")
                except subprocess.CalledProcessError as e:
                    print(f"Error updating {library_name} to version {library_version}: {e}")

        except pkg_resources.DistributionNotFound:
            # The library is not installed, so install it
            try:
                _pip_install(library)
                # subprocess.run(["pip", "install", library], check=True)
                print(f"{library} has been installed.")
            except subprocess.CalledProcessError as e:
                print(f"Error installing {library}: {e}")

    print("Bootstrapping is done, Relevant instrumentation libraries associated with existing packages. \n")


from opentelemetry import trace
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.sdk.trace.export import BatchSpanProcessor

# mw_agent_target = os.environ.get('MW_AGENT_SERVICE', '127.0.0.1')


def collect_traces():
    provider = trace.get_tracer_provider()
    if not provider:
        trace.set_tracer_provider(TracerProvider())
    provider.add_span_processor(BatchSpanProcessor(OTLPSpanExporter()))


def record_error(error):
    span = trace.get_current_span()
    if span:
        span.record_exception(error)
        span.set_status(trace.Status(trace.StatusCode.ERROR, str(error)))


# def set_attribute(name, value):
#     if type(name) is not str:
#         print("name must be a string")
#         return
#     if type(value) is not str:
#         print("value must be a string")
#         return
#     span = trace.get_current_span()
#     span.set_attribute(name, value)



import sys

_commands = {}

_imports = [
    "run"
]


def default_imports():
    for name in _imports:
        module_name = "%s.%s" % (__name__, name)
        __import__(module_name)


def command(name, options="", description="", hidden=False):
    def wrapper(callback):
        callback.name = name
        callback.options = options
        callback.description = description
        callback.hidden = hidden
        _commands[name] = callback
        return callback

    return wrapper


@command("help", "[command]", hidden=True)
def help(args):
    if not args:
        print("Usage: middleware-apm command [options]")
        print()
        print("Type 'middleware-apm help <command>'", end="")
        print("for help on a specific command.")
        print()
        print("Available commands are:")

        commands = sorted(_commands.keys())
        for name in commands:
            details = _commands[name]
            if not details.hidden:
                print(" ", name)
    else:
        name = args[0]
        if name not in _commands:
            print("Unknown command '%s'." % name, end=" ")
            print("Type 'middleware-apm help' for usage.")
        else:
            details = _commands[name]
            print("Usage: middleware-apm %s %s" % (name, details.options))
            if details.description:
                print()
                description = details.description
                print(description)


def main() -> None:
    try:
        if len(sys.argv) > 1:
            command = sys.argv[1]
        else:
            command = "help"

        callback = _commands[command]

    except Exception:
        print("Unknown command '%s'." % command, end="")
        print("Type 'middleware-apm help' for usage.")
        sys.exit(1)

    callback(sys.argv[2:])


default_imports()

if __name__ == "__main__":
    main()



import sys
import setuptools

python_version = sys.version_info[:2]
assert python_version >= (3,8,), "Middleware APM supports: Python 3.8+"

try:
    with open("requirements.txt", "r") as f:
        requirements = f.read().splitlines()
except IOError:
    requirements = []

packages = [
    "scripts",
    "middleware",
    "middleware.installer",
    "middleware.lib",
]

setuptools.setup(
    name="middleware-apm",
    version="0.5.0rc2",
    install_requires=requirements,
    author="middleware-dev",
    maintainer="middleware-dev",
    license="Apache-2.0",
    description="Middleware's APM tool enables Python developers to effortlessly monitor their applications, gathering distributed tracing, metrics, logs, and profiling data for valuable insights and performance optimization. Install the Middleware Host agent and integrate the APM package to leverage its powerful functionalities.",
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    packages=packages,
    python_requires=">=3.8",
    url = "https://docs.middleware.io/docs/apm-configuration/python",
    project_urls={"Source": "https://github.com/middleware-labs/agent-apm-python.git"},
    entry_points={
        'console_scripts': [
            'middleware-apm = middleware.installer:main',
            'middleware-instrument = opentelemetry.instrumentation.auto_instrumentation:run',
            'middleware-bootstrap = opentelemetry.instrumentation.bootstrap:run',
        ],
    },
)